// This programm allows the Arduino to act as a native MIDI device which sends out CC messages on channel 0. 
// In the sketch, some smoothing is done to prevent an overflow of data due to the varying sensitivy of potentiometers.
// Remember that flashing your Arduino with the right software is required to turn your Arduino in a recognizable MIDI device! 

//Short description_______________________________________________________________________________________________________________________________
// This deliverable for the academic study ‘Creative Programming’ shows an Audio Visualizer which feeds on the internal sound flowing
// through your computer! Various settings can be changed by the Arduino MIDI controller or literally any other device capable of sending
// MIDI messages (See the Maschine MK2 drum controller in the video). If you do not have access of a MIDI device, hit the ’s’ key to get
// a preview of the possibilities of the Audio Visualizer. 

//How it works____________________________________________________________________________________________________________________________________
// Processing uses the Minim library to analyse the incoming audio for amplitude, spectrum frequencies and beat detection. Certain parameters 
// are linked to these analyses to make the sound appear visually on your screen. With the Arduino (or other MIDI devices) it is possible to 
// change certain parameters to make the Audio Visualizer more appealing.

// © Copyright by Bas van Straaten

// Define some global variables & inputs
#define NUM_AI 5 // Number of potentiometers
#define ANALOGUE_PIN_ORDER A4, A3, A2, A1, A0
#define THRESSHOLD 4 // Define the responsivity of the knobs
#define TIMEOUT 100 // Define the responsivity of the knobs
#define LED 9

// Do not modify the following variables!
boolean midiout;
boolean analogueInputChanging[NUM_AI];
unsigned long analogueInputTimer[NUM_AI];
int analogueInputMapping[NUM_AI] = {ANALOGUE_PIN_ORDER};
int analogValue;
int analogueInputs[NUM_AI];
int analogLastValue[NUM_AI];
int previousMillis = 0;
byte value[NUM_AI];
byte lastValue[NUM_AI];

//____________________________________________________________________________
void setup() {
  Serial.begin(31250); // This serial baudrate is used to communicate with MIDI-devices
  pinMode(LED, OUTPUT);
  for (byte i = 0; i < NUM_AI; i++) {
    pinMode(analogueInputMapping[i], INPUT);
    analogueInputChanging[i] = false;
    analogueInputTimer[i] = 0;
  }
}

//____________________________________________________________________________
void loop() {
  for (byte i = 0; i < NUM_AI; i++)  {
    analogValue = analogRead(analogueInputMapping[i]);
    if (abs(analogValue - analogLastValue[i]) > THRESSHOLD) { // Check if the knob has moved more than the thresshold value
      analogueInputChanging[i] = true; 
      analogueInputTimer[i] = millis();
      analogLastValue[i] = analogValue;
    }
    else if (millis() - analogueInputTimer[i] > TIMEOUT) { // If the knob has not moved, set the 'analogInputChanging' to false
      analogueInputChanging[i] = false;
    }
    if (analogueInputChanging[i] == true) { // If the knob moved, map the max/min value to a MIDI value
      value[i] = map(analogRead(analogueInputMapping[i]), 1023, 0, 0, 127);
      if (value[i] != lastValue[i]) { // If the MIDI value has not been sent just before, send it
        sendMidi(0xB0, 0 + i, value[i]);
        lastValue[i] = value[i];
        midiout = true;
      }
    }
  }
  ledStatus();
}

//____________________________________________________________________________
void sendMidi(int statusbyte, int data1, int data2) { // Write the MIDI messages
  Serial.write(statusbyte);
  Serial.write(data1);
  Serial.write(data2);
}

//____________________________________________________________________________
void ledStatus() { // Turn on the led as a indication for the MIDI messages
  if (midiout == true) {
    digitalWrite(LED, HIGH);
  }
  int currentMillis = millis();
  if (currentMillis <= 10) {
    previousMillis = currentMillis;
  }
  if (currentMillis - previousMillis > 50) {
    previousMillis = currentMillis;
    digitalWrite(LED, LOW);
    midiout = false;
  }
}
